@@ -1,23 +1,22 @@
-"""This module encapsulates a document stored in a GNUmed database.
-
-@copyright: GPL v2 or later
-"""
+"""This module encapsulates a document stored in a GNUmed database."""
 #============================================================
-__version__ = "$Revision: 1.118 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
+__license__ = "GPL v2 or later"
 
 import sys, os, shutil, os.path, types, time, logging
-from cStringIO import StringIO
-from pprint import pprint
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-from Gnumed.pycommon import gmExceptions, gmBusinessDBObject, gmPG2, gmTools, gmMimeLib
+from Gnumed.pycommon import gmExceptions
+from Gnumed.pycommon import gmBusinessDBObject
+from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmMimeLib
+from Gnumed.pycommon import gmDateTime
 
 
 _log = logging.getLogger('gm.docs')
-_log.info(__version__)
 
 MUGSHOT=26
 DOCUMENT_TYPE_VISUAL_PROGRESS_NOTE = u'visual progress note'
@@ -91,13 +90,14 @@ class cDocumentFolder:
 		return prescription
 	#--------------------------------------------------------
 	def get_latest_mugshot(self):
-		cmd = u"select pk_obj from blobs.v_latest_mugshot where pk_patient=%s"
+		cmd = u"SELECT pk_obj FROM blobs.v_latest_mugshot WHERE pk_patient = %s"
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_patient]}])
 		if len(rows) == 0:
 			_log.info('no mugshots available for patient [%s]' % self.pk_patient)
 			return None
-		mugshot = cDocumentPart(aPK_obj=rows[0][0])
-		return mugshot
+		return cDocumentPart(aPK_obj = rows[0][0])
+
+	latest_mugshot = property(get_latest_mugshot, lambda x:x)
 	#--------------------------------------------------------
 	def get_mugshot_list(self, latest_only=True):
 		if latest_only:
@@ -156,7 +156,22 @@ class cDocumentFolder:
 			encounter = encounter
 		)
 	#--------------------------------------------------------
-	def get_documents(self, doc_type=None, episodes=None, encounter=None):
+	def get_unsigned_documents(self):
+		args = {'pat': self.pk_patient}
+		cmd = _sql_fetch_document_fields % u"""
+			pk_doc IN (
+				SELECT DISTINCT ON (b_vo.pk_doc) b_vo.pk_doc
+				FROM blobs.v_obj4doc_no_data b_vo
+				WHERE
+					pk_patient = %(pat)s
+						AND
+					reviewed IS FALSE
+			)
+			ORDER BY clin_when DESC"""
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+		return [ cDocument(row = {'pk_field': 'pk_doc', 'idx': idx, 'data': r}) for r in rows ]
+	#--------------------------------------------------------
+	def get_documents(self, doc_type=None, episodes=None, encounter=None, order_by=None, exclude_unsigned=False):
 		"""Return list of documents."""
 
 		args = {
@@ -180,10 +195,18 @@ class cDocumentFolder:
 		if encounter is not None:
 			where_parts.append(u'pk_encounter = %(enc)s')
 
-		cmd = u"%s\nORDER BY clin_when" % (_sql_fetch_document_fields % u' AND '.join(where_parts))
+		if exclude_unsigned:
+			where_parts.append(u'pk_doc IN (SELECT b_vo.pk_doc FROM blobs.v_obj4doc_no_data b_vo WHERE b_vo.pk_patient = %(pat)s AND b_vo.reviewed IS TRUE)')
+
+		if order_by is None:
+			order_by = u'ORDER BY clin_when'
+
+		cmd = u"%s\n%s" % (_sql_fetch_document_fields % u' AND '.join(where_parts), order_by)
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
 
 		return [ cDocument(row = {'pk_field': 'pk_doc', 'idx': idx, 'data': r}) for r in rows ]
+
+	documents = property(get_documents, lambda x:x)
 	#--------------------------------------------------------
 	def add_document(self, document_type=None, encounter=None, episode=None):
 		return create_document(document_type = document_type, encounter = encounter, episode = episode)
@@ -195,26 +218,30 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 
 	_cmd_fetch_payload = _sql_fetch_document_part_fields % u"pk_obj = %s"
 	_cmds_store_payload = [
-		u"""update blobs.doc_obj set
+		u"""UPDATE blobs.doc_obj SET
 				seq_idx = %(seq_idx)s,
 				comment = gm.nullify_empty_string(%(obj_comment)s),
 				filename = gm.nullify_empty_string(%(filename)s),
-				fk_intended_reviewer = %(pk_intended_reviewer)s
-			where
-				pk=%(pk_obj)s and
-				xmin=%(xmin_doc_obj)s""",
-		u"""select xmin_doc_obj from blobs.v_obj4doc_no_data where pk_obj = %(pk_obj)s"""
+				fk_intended_reviewer = %(pk_intended_reviewer)s,
+				fk_doc = %(pk_doc)s
+			WHERE
+				pk = %(pk_obj)s
+					AND
+				xmin = %(xmin_doc_obj)s
+			RETURNING
+				xmin AS xmin_doc_obj"""
 	]
 	_updatable_fields = [
 		'seq_idx',
 		'obj_comment',
 		'pk_intended_reviewer',
-		'filename'
+		'filename',
+		'pk_doc'
 	]
 	#--------------------------------------------------------
 	# retrieve data
 	#--------------------------------------------------------
-	def export_to_file(self, aTempDir = None, aChunkSize = 0, filename=None):
+	def export_to_file(self, aChunkSize=0, filename=None, target_mime=None, target_extension=None, ignore_conversion_problems=False):
 
 		if self._payload[self._idx['size']] == 0:
 			return None
@@ -230,8 +257,7 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 			# get unique filename
 			filename = gmTools.get_unique_filename (
 				prefix = 'gm-doc_obj-page_%s-' % self._payload[self._idx['seq_idx']],
-				suffix = suffix,
-				tmp_dir = aTempDir
+				suffix = suffix
 			)
 
 		success = gmPG2.bytea2file (
@@ -244,10 +270,33 @@ class cDocumentPart(gmBusinessDBObject.cBusinessDBObject):
 			data_size = self._payload[self._idx['size']]
 		)
 
-		if success:
+		if not success:
+			return None
+
+		if target_mime is None:
 			return filename
 
-		return None
+		if target_extension is None:
+			target_extension = gmMimeLib.guess_ext_by_mimetype(mimetype = target_mime)
+
+		target_fname = gmTools.get_unique_filename (
+			prefix = 'gm-doc_obj-page_%s-converted-' % self._payload[self._idx['seq_idx']],
+			suffix = target_extension
+		)
+		_log.debug('attempting conversion: [%s] -> [<%s>:%s]', filename, target_mime, target_fname)
+		if gmMimeLib.convert_file (
+			filename = filename,
+			target_mime = target_mime,
+			target_filename = target_fname
+		):
+			return target_fname
+
+		_log.warning('conversion failed')
+		if not ignore_conversion_problems:
+			return None
+
+		_log.warning('programmed to ignore conversion problems, hoping receiver can handle [%s]', filename)
+		return filename
 	#--------------------------------------------------------
 	def get_reviews(self):
 		cmd = u"""
@@ -281,7 +330,7 @@ order by
 			return False
 
 		gmPG2.file2bytea (
-			query = u"UPDATE blobs.doc_obj SET data=%(data)s::bytea WHERE pk=%(pk)s",
+			query = u"UPDATE blobs.doc_obj SET data = %(data)s::bytea WHERE pk = %(pk)s",
 			filename = fname,
 			args = {'pk': self.pk_obj}
 		)
@@ -358,9 +407,9 @@ where
 		self._is_modified = True
 		self.save_payload()
 	#--------------------------------------------------------
-	def display_via_mime(self, tmpdir=None, chunksize=0, block=None):
+	def display_via_mime(self, chunksize=0, block=None):
 
-		fname = self.export_to_file(aTempDir = tmpdir, aChunkSize = chunksize)
+		fname = self.export_to_file(aChunkSize = chunksize)
 		if fname is None:
 			return False, ''
 
@@ -369,19 +418,55 @@ where
 			return False, msg
 
 		return True, ''
+	#--------------------------------------------------------
+	def format(self):
+		txt = _('%s document part                 [#%s]\n') % (
+			gmTools.bool2str (
+				boolean = self._payload[self._idx['reviewed']],
+				true_str = _('Reviewed'),
+				false_str = _('Unreviewed')
+			),
+			self._payload[self._idx['pk_obj']]
+		)
+
+		f_ext = u''
+		if self._payload[self._idx['filename']] is not None:
+			f_ext = os.path.splitext(self._payload[self._idx['filename']])[1].strip('.').strip()
+		if f_ext != u'':
+			f_ext = u'.' + f_ext.upper() + u' '
+		txt += _(' Part %s: %s %s(%s Bytes)\n') % (
+			self._payload[self._idx['seq_idx']],
+			gmTools.size2str(self._payload[self._idx['size']]),
+			f_ext,
+			self._payload[self._idx['size']]
+		)
+
+		if self._payload[self._idx['filename']] is not None:
+			txt += _(' Filename: %s\n') % self._payload[self._idx['filename']]
+
+		if self._payload[self._idx['obj_comment']] is not None:
+			txt += u'\n%s\n' % self._payload[self._idx['obj_comment']]
+
+		return txt
+#------------------------------------------------------------
+def delete_document_part(part_pk=None, encounter_pk=None):
+	cmd = u"select blobs.delete_document_part(%(pk)s, %(enc)s)"
+	args = {'pk': part_pk, 'enc': encounter_pk}
+	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
+	return
 #============================================================
 _sql_fetch_document_fields = u"""
-	SELECT
-		*,
-	COALESCE (
-		(SELECT array_agg(seq_idx) FROM blobs.doc_obj b_do WHERE b_do.fk_doc = b_vdm.pk_doc),
-		ARRAY[]::integer[]
-	)
-		AS seq_idx_list
-	FROM
-		blobs.v_doc_med b_vdm
-	WHERE
-		%s
+		SELECT
+			*,
+		COALESCE (
+			(SELECT array_agg(seq_idx) FROM blobs.doc_obj b_do WHERE b_do.fk_doc = b_vdm.pk_doc),
+			ARRAY[]::integer[]
+		)
+			AS seq_idx_list
+		FROM
+			blobs.v_doc_med b_vdm
+		WHERE
+			%s
 """
 
 class cDocument(gmBusinessDBObject.cBusinessDBObject):
@@ -447,7 +532,7 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 		return True
 	#--------------------------------------------------------
 	def _get_parts(self):
-		cmd = _sql_fetch_document_part_fields % u"pk_doc = %s"
+		cmd = _sql_fetch_document_part_fields % u"pk_doc = %s ORDER BY seq_idx"
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': [self.pk_obj]}], get_col_idx = True)
 		return [ cDocumentPart(row = {'pk_field': 'pk_obj', 'idx': idx, 'data': r}) for r in rows ]
 
@@ -513,14 +598,11 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 	def export_parts_to_files(self, export_dir=None, chunksize=0):
 		fnames = []
 		for part in self.parts:
-			# FIXME: add guess_extension_from_mimetype
-			fname = os.path.basename(gmTools.coalesce (
-				part['filename'],
-				u'%s%s%s_%s' % (part['l10n_type'], gmTools.coalesce(part['ext_ref'], '-', '-%s-'), _('part'), part['seq_idx'])
-			))
+			fname = part.export_to_file(aChunkSize = chunksize)
 			if export_dir is not None:
-				fname = os.path.join(export_dir, fname)
-			fnames.append(part.export_to_file(aChunkSize = chunksize, filename = fname))
+				shutil.move(fname, export_dir)
+				fname = os.path.join(export_dir, os.path.split(fname)[1])
+			fnames.append(fname)
 		return fnames
 	#--------------------------------------------------------
 	def _get_has_unreviewed_parts(self):
@@ -554,6 +636,34 @@ class cDocument(gmBusinessDBObject.cBusinessDBObject):
 				_log.error(str(data))
 				return False
 		return True
+	#--------------------------------------------------------
+	def format(self):
+		part_count = len(self._payload[self._idx['seq_idx_list']])
+		if part_count == 0:
+			parts = _('no parts')
+		elif part_count == 1:
+			parts = _('1 part')
+		else:
+			parts = _('%s parts') % part_count
+		txt = _(
+			'%s (%s)   #%s\n'
+			'\n'
+			' Created: %s\n'
+			' Episode: %s\n'
+			'%s'
+			'%s'
+			'%s'
+		) % (
+			self._payload[self._idx['l10n_type']],
+			parts,
+			self._payload[self._idx['pk_doc']],
+			gmDateTime.pydt_strftime(self._payload[self._idx['clin_when']], format = '%Y %b %d', accuracy = gmDateTime.acc_days),
+			self._payload[self._idx['episode']],
+			gmTools.coalesce(self._payload[self._idx['health_issue']], u'', _(' Health issue: %s\n')),
+			gmTools.coalesce(self._payload[self._idx['ext_ref']], u'', _(' External reference: %s\n')),
+			gmTools.coalesce(self._payload[self._idx['comment']], u'', u' %s')
+		)
+		return txt
 #------------------------------------------------------------
 def create_document(document_type=None, encounter=None, episode=None):
 	"""Returns new document instance or raises an exception.
@@ -581,34 +691,33 @@ def create_document(document_type=None, encounter=None, episode=None):
 	doc = cDocument(aPK_obj = rows[0][0])
 	return doc
 #------------------------------------------------------------
-def search_for_document(patient_id=None, type_id=None):
-	"""Searches for documents with the given patient and type ID.
-
-	No type ID returns all documents for the patient.
-	"""
-	# sanity checks
+def search_for_documents(patient_id=None, type_id=None, external_reference=None):
+	"""Searches for documents with the given patient and type ID."""
 	if patient_id is None:
 		raise ValueError('need patient id to search for document')
 
-	args = {'pat_id': patient_id, 'type_id': type_id}
-	if type_id is None:
-		cmd = u"SELECT pk_doc from blobs.v_doc_med WHERE pk_patient = %(pat_id)s"
-	else:
-		cmd = u"SELECT pk_doc from blobs.v_doc_med WHERE pk_patient = %(pat_id)s and pk_type = %(type_id)s"
+	args = {'pat_id': patient_id, 'type_id': type_id, 'ref': external_reference}
+	where_parts = [u'pk_patient = %(pat_id)s']
 
-	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}])
+	if type_id is not None:
+		where_parts.append(u'pk_type = %(type_id)s')
 
-	docs = []
-	for row in rows:
-		docs.append(cDocument(row[0]))
-	return docs
+	if external_reference is not None:
+		where_parts.append(u'ext_ref = %(ref)s')
+
+	cmd = _sql_fetch_document_fields % u' AND '.join(where_parts)
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = True)
+	return [ cDocument(row = {'data': r, 'idx': idx, 'pk_field': 'pk_doc'}) for r in rows ]
 #------------------------------------------------------------
 def delete_document(document_id=None, encounter_id=None):
-	# will cascade to doc_obj and doc_desc
-	cmd = u"select blobs.delete_document(%(pk)s, %(enc)s)"
+	# cascades to doc_obj and doc_desc but not bill.bill
+	cmd = u"SELECT blobs.delete_document(%(pk)s, %(enc)s)"
 	args = {'pk': document_id, 'enc': encounter_id}
-	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}])
-	return
+	rows, idx = gmPG2.run_rw_queries(queries = [{'cmd': cmd, 'args': args}], return_data = True)
+	if not rows[0][0]:
+		_log.error('cannot delete document [%s]', document_id)
+		return False
+	return True
 #------------------------------------------------------------
 def reclassify_documents_by_type(original_type=None, target_type=None):
 
@@ -670,7 +779,6 @@ class cDocumentType(gmBusinessDBObject.cBusinessDBObject):
 			return False
 
 		return self.refetch_payload()
-
 #------------------------------------------------------------
 def get_document_types():
 	rows, idx = gmPG2.run_ro_queries (
@@ -679,14 +787,24 @@ def get_document_types():
 	)
 	doc_types = []
 	for row in rows:
-		row_def = {
-			'pk_field': 'pk_doc_type',
-			'idx': idx,
-			'data': row
-		}
+		row_def = {'pk_field': 'pk_doc_type', 'idx': idx, 'data': row}
 		doc_types.append(cDocumentType(row = row_def))
 	return doc_types
 #------------------------------------------------------------
+def get_document_type_pk(document_type=None):
+	args = {'typ': document_type.strip()}
+
+	cmd = u'SELECT pk FROM blobs.doc_type WHERE name = %(typ)s'
+	rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+	if len(rows) == 0:
+		cmd = u'SELECT pk FROM blobs.doc_type WHERE _(name) = %(typ)s'
+		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': args}], get_col_idx = False)
+
+	if len(rows) == 0:
+		return None
+
+	return rows[0]['pk']
+#------------------------------------------------------------
 def create_document_type(document_type=None):
 	# check for potential dupes:
 	cmd = u'select pk from blobs.doc_type where name = %s'
@@ -694,13 +812,9 @@ def create_document_type(document_type=None):
 		queries = [{'cmd': cmd, 'args': [document_type]}]
 	)
 	if len(rows) == 0:
-		cmd1 = u"insert into blobs.doc_type (name) values (%s)"
-		cmd2 = u"select currval('blobs.doc_type_pk_seq')"
+		cmd1 = u"INSERT INTO blobs.doc_type (name) VALUES (%s) RETURNING pk"
 		rows, idx = gmPG2.run_rw_queries (
-			queries = [
-				{'cmd': cmd1, 'args': [document_type]},
-				{'cmd': cmd2}
-			],
+			queries = [{'cmd': cmd1, 'args': [document_type]}],
 			return_data = True
 		)
 	return cDocumentType(aPK_obj = rows[0][0])
@@ -770,7 +884,7 @@ if __name__ == '__main__':
 		print "testing document import"
 		print "-----------------------"
 
-		docs = search_for_document(patient_id=12)
+		docs = search_for_documents(patient_id=12)
 		doc = docs[0]
 		print "adding to doc:", doc
 
