@@ -162,7 +162,7 @@ class cPatientSearcher_SQL:
 		"""Compose queries if search term seems unambigous."""
 		queries = []
 
-		raw = raw.rstrip(u',').rstrip(u';')
+		raw = raw.strip().rstrip(u',').rstrip(u';').strip()
 
 		# "<digits>" - GNUmed patient PK or DOB
 		if regex.match(u"^(\s|\t)*\d+(\s|\t)*$", raw, flags = regex.LOCALE | regex.UNICODE):
@@ -364,7 +364,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 	#--------------------------------------------------------
 	# queries for DE
 	#--------------------------------------------------------
-	def _generate_queries_de(self, search_term = None):
+	def _generate_queries_de(self, search_term=None):
 
 		if search_term is None:
 			return []
@@ -377,7 +377,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 		# no we don't
 		_log.debug('[%s]: not a search term with a "suggestive" structure' % search_term)
 
-		search_term = search_term.strip(u',').strip(u';')
+		search_term = search_term.strip().strip(u',').strip(u';').strip()
 		normalized = self._normalize_soundalikes(search_term)
 
 		queries = []
@@ -395,7 +395,16 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 		SELECT vbp.*, %s::text AS match_type FROM dem.v_basic_person vbp, dem.names n WHERE vbp.pk_identity = n.id_identity and lower(n.firstnames) ~* lower(%s)
 	) union all (
 		-- anywhere in name
-		SELECT vbp.*, %s::text AS match_type FROM dem.v_basic_person vbp, dem.names n WHERE vbp.pk_identity = n.id_identity and lower(n.firstnames || n.lastnames || coalesce(n.preferred, '')) ~* lower(%s)
+		SELECT
+			vbp.*,
+			%s::text AS match_type
+		FROM
+			dem.v_basic_person vbp,
+			dem.names n
+		WHERE
+			vbp.pk_identity = n.id_identity
+				AND
+			lower(n.firstnames || ' ' || n.lastnames || ' ' || coalesce(n.preferred, '')) ~* lower(%s)
 	)) AS super_list ORDER BY lastnames, firstnames, dob
 ) AS sorted_list"""
 			tmp = normalized.strip()
@@ -423,11 +432,16 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 		if len(parts_list) == 1:
 			# re-split on whitespace
 			sub_parts_list = regex.split(u"\s*|\t*", normalized)
+			# ignore empty parts
+			sub_parts_list = [ p.strip() for p in sub_parts_list if p.strip() != u'' ]
 
 			# parse into name/date parts
 			date_count = 0
 			name_parts = []
 			for part in sub_parts_list:
+				# skip empty parts
+				if part.strip() == u'':
+					continue
 				# any digit signifies a date
 				# FIXME: what about "<40" ?
 				if regex.search(u"\d", part, flags = regex.LOCALE | regex.UNICODE):
@@ -460,7 +474,20 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 					})
 					# name parts anywhere in name - third order query ...
 					queries.append ({
-						'cmd': u"SELECT DISTINCT ON (id_identity) vbp.*, %s::text AS match_type FROM dem.v_basic_person vbp, dem.names n WHERE vbp.pk_identity = n.id_identity and lower(n.firstnames || n.lastnames) ~* lower(%s) AND lower(n.firstnames || n.lastnames) ~* lower(%s)",
+						'cmd': u"""SELECT DISTINCT ON (id_identity)
+									vbp.*,
+									%s::text AS match_type
+								FROM
+									dem.v_basic_person vbp,
+									dem.names n
+								WHERE
+									vbp.pk_identity = n.id_identity
+										AND
+									-- name_parts[0]
+									lower(n.firstnames || ' ' || n.lastnames) ~* lower(%s)
+										AND
+									-- name_parts[1]
+									lower(n.firstnames || ' ' || n.lastnames) ~* lower(%s)""",
 						'args': [_('name'), name_parts[0], name_parts[1]]
 					})
 					return queries
@@ -492,7 +519,21 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 					})
 					# name parts anywhere in name - third order query ...
 					queries.append ({
-						'cmd': u"SELECT DISTINCT ON (id_identity) vbp.*, %s::text AS match_type FROM dem.v_basic_person vbp, dem.names n WHERE vbp.pk_identity = n.id_identity and lower(n.firstnames || n.lastnames) ~* lower(%s) AND lower(n.firstnames || n.lastnames) ~* lower(%s) AND dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %s::timestamp with time zone)",
+						'cmd': u"""SELECT DISTINCT ON (id_identity)
+									vbp.*,
+									%s::text AS match_type
+								FROM
+									dem.v_basic_person vbp,
+									dem.names n
+								WHERE
+									vbp.pk_identity = n.id_identity
+										AND
+									lower(n.firstnames || ' ' || n.lastnames) ~* lower(%s)
+										AND
+									lower(n.firstnames || ' ' || n.lastnames) ~* lower(%s)
+										AND
+									dem.date_trunc_utc('day'::text, dob) = dem.date_trunc_utc('day'::text, %s::timestamp with time zone)
+						""",
 						'args': [_('name, date of birth'), name_parts[0], name_parts[1], date_part.replace(u',', u'.')]
 					})
 					return queries
@@ -511,6 +552,8 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 			name_parts = []
 			name_count = 0
 			for part in parts_list:
+				if part.strip() == u'':
+					continue
 				# any digits ?
 				if regex.search(u"\d+", part, flags = regex.LOCALE | regex.UNICODE):
 					# FIXME: parse out whitespace *not* adjacent to a *word*
@@ -545,7 +588,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 				})
 				# or even substrings anywhere in name
 				where_parts.append ({
-					'conditions': u"lower(firstnames || lastnames) ~* lower(%s) OR lower(firstnames || lastnames) ~* lower(%s)",
+					'conditions': u"lower(firstnames || ' ' || lastnames) ~* lower(%s) OR lower(firstnames || ' ' || lastnames) ~* lower(%s)",
 					'args': [_('name'), name_parts[0][0], name_parts[0][1]]
 				})
 
@@ -571,7 +614,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 				})
 				# or even substrings anywhere in name
 				where_parts.append ({
-					'conditions': u"lower(firstnames || lastnames) ~* lower(%s) AND lower(firstnames || lastnames) ~* lower(%s)",
+					'conditions': u"lower(firstnames || ' ' || lastnames) ~* lower(%s) AND lower(firstnames || ' ' || lastnames) ~* lower(%s)",
 					'args': [_('name'), ' '.join(name_parts[0]), ' '.join(name_parts[1])]
 				})
 
@@ -581,7 +624,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 				if len(name_parts) == 1:
 					for part in name_parts[0]:
 						where_parts.append ({
-							'conditions': u"lower(firstnames || lastnames) ~* lower(%s)",
+							'conditions': u"lower(firstnames || ' ' || lastnames) ~* lower(%s)",
 							'args': [_('name'), part]
 						})
 				else:
@@ -590,7 +633,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 						tmp.append(' '.join(part))
 					for part in tmp:
 						where_parts.append ({
-							'conditions': u"lower(firstnames || lastnames) ~* lower(%s)",
+							'conditions': u"lower(firstnames || ' ' || lastnames) ~* lower(%s)",
 							'args': [_('name'), part]
 						})
 
@@ -643,7 +686,7 @@ SELECT DISTINCT ON (pk_identity) * FROM (
 		args = []
 		# FIXME: split on more than just ' '
 		for arg in search_term.strip().split():
-			where_clause += u" AND lower(coalesce(vbp.title, '') || vbp.firstnames || vbp.lastnames) ~* lower(%s)"
+			where_clause += u" AND lower(coalesce(vbp.title, '') || ' ' || vbp.firstnames || ' ' || vbp.lastnames) ~* lower(%s)"
 			args.append(arg)
 
 		query = u"""
