@@ -74,6 +74,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if self._TCTRL_xxx.GetValue().strip() == u'':
 			validity = False
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_xxx, valid = False)
+			gmDispatcher.send(signal = 'statustext', msg = _('No entry in field xxx.'))
 			self._TCTRL_xxx.SetFocus()
 		else:
 			self.display_tctrl_as_valid(tctrl = self._TCTRL_xxx, valid = True)
@@ -81,6 +82,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		if self._PRW_xxx.GetData() is None:
 			validity = False
 			self._PRW_xxx.display_as_valid(False)
+			gmDispatcher.send(signal = 'statustext', msg = _('No entry in field xxx.'))
 			self._PRW_xxx.SetFocus()
 		else:
 			self._PRW_xxx.display_as_valid(True)
@@ -88,6 +90,10 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		return validity
 	#----------------------------------------------------------------
 	def _save_as_new(self):
+
+		# remove when implemented:
+		return False
+
 		# save the data as a new instance
 		data = gmXXXX.create_xxxx()
 
@@ -104,6 +110,10 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 		return True
 	#----------------------------------------------------------------
 	def _save_as_update(self):
+
+		# remove when implemented:
+		return False
+
 		# update self.data and save the changes
 		self.data[''] = self._TCTRL_xxx.GetValue().strip()
 		self.data[''] = self._PRW_xxx.GetData()
@@ -217,8 +227,7 @@ class cXxxEAPnl(wxgXxxEAPnl.wxgXxxEAPnl, gmEditArea.cGenericEditAreaMixin):
 			raise ValueError('[%s] <mode> must be in %s' % (self.__class__.__name__, edit_area_modes))
 	#----------------------------------------------------------------
 	def display_tctrl_as_valid(self, tctrl=None, valid=None):
-		tctrl.SetBackgroundColour(self.__tctrl_validity_colors[valid])
-		tctrl.Refresh()
+		self.display_ctrl_as_valid(ctrl = tctrl, valid = valid)
 	#----------------------------------------------------------------
 	def display_ctrl_as_valid(self, ctrl=None, valid=None):
 		ctrl.SetBackgroundColour(self.__tctrl_validity_colors[valid])
@@ -280,6 +289,10 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 
 		self._adjust_clear_revert_buttons()
 
+		# attach listener
+		self._TCTRL_status.SetValue('')
+		gmDispatcher.connect(signal = u'statustext', receiver = self._on_set_statustext)
+
 		# redraw layout
 		#self.Layout()
 		main_szr = self.GetSizer()
@@ -289,6 +302,16 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 
 		self._PNL_ea.refresh()
 	#--------------------------------------------------------
+	def _on_set_statustext(self, msg=None, loglevel=None, beep=True):
+		if msg is None:
+			self._TCTRL_status.SetValue('')
+			return
+		if msg.strip() == u'':
+			self._TCTRL_status.SetValue('')
+			return
+		self._TCTRL_status.SetValue(msg)
+		return
+	#--------------------------------------------------------
 	def _adjust_clear_revert_buttons(self):
 		if self._PNL_ea.data is None:
 			self._BTN_clear.Enable(True)
@@ -303,6 +326,7 @@ class cGenericEditAreaDlg2(wxgGenericEditAreaDlg2.wxgGenericEditAreaDlg2):
 	#--------------------------------------------------------
 	def _on_save_button_pressed(self, evt):
 		if self._PNL_ea.save():
+			gmDispatcher.disconnect(signal = u'statustext', receiver = self._on_set_statustext)
 			if self.IsModal():
 				self.EndModal(wx.ID_OK)
 			else:
@@ -428,15 +452,15 @@ gmSECTION_RECALLS = 12
 
 richards_blue = wx.Colour(0,0,131)
 richards_aqua = wx.Colour(0,194,197)
-richards_dark_gray = wx.Color(131,129,131)
-richards_light_gray = wx.Color(255,255,255)
-richards_coloured_gray = wx.Color(131,129,131)
+richards_dark_gray = wx.Colour(131,129,131)
+richards_light_gray = wx.Colour(255,255,255)
+richards_coloured_gray = wx.Colour(131,129,131)
 
 
 CONTROLS_WITHOUT_LABELS =['wxTextCtrl', 'cEditAreaField', 'wx.SpinCtrl', 'gmPhraseWheel', 'wx.ComboBox'] 
 
 def _decorate_editarea_field(widget):
-	widget.SetForegroundColour(wx.Color(255, 0, 0))
+	widget.SetForegroundColour(wx.Colour(255, 0, 0))
 	widget.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD, False, ''))
 #====================================================================
 class cEditAreaPopup(wx.Dialog):
@@ -539,7 +563,7 @@ class cEditArea2(wx.Panel):
 			size = size,
 			style = style | wx.TAB_TRAVERSAL
 		)
-		self.SetBackgroundColour(wx.Color(222,222,222))
+		self.SetBackgroundColour(wx.Colour(222,222,222))
 
 		self.data = None		# a placeholder for opaque data
 		self.fields = {}
@@ -704,7 +728,7 @@ class cEditArea2(wx.Panel):
 			positions.sort()
 			for pos in positions:
 				field, weight = self.fields[line][pos]
-#				field.SetBackgroundColour(wx.Color(222,222,222))
+#				field.SetBackgroundColour(wx.Colour(222,222,222))
 				szr_line.Add(field, weight, wx.EXPAND)
 			szr_main_fgrid.Add(szr_line, flag=wx.GROW | wx.ALIGN_LEFT)
 
@@ -786,7 +810,7 @@ class cEditArea(wx.Panel):
 
 		# init main background panel
 		wx.Panel.__init__(self, parent, id, pos=pos, size=size, style=wx.NO_BORDER | wx.TAB_TRAVERSAL)
-		self.SetBackgroundColour(wx.Color(222,222,222))
+		self.SetBackgroundColour(wx.Colour(222,222,222))
 
 		self.data = None
 		self.fields = {}
@@ -875,7 +899,7 @@ class cEditArea(wx.Panel):
 		return hszr_prompts
 	#----------------------------------------------------------------
 	def __generate_fields(self):
-		self.fields_pnl.SetBackgroundColour(wx.Color(222,222,222))
+		self.fields_pnl.SetBackgroundColour(wx.Colour(222,222,222))
 		# rows, cols, hgap, vgap
 		vszr = wx.BoxSizer(wx.VERTICAL)
 		lines = self.fields.keys()
@@ -1137,7 +1161,7 @@ class gmEditArea(cEditArea):
 	def __make_editing_area(self):
 		# make edit fields
 		fields_pnl = wx.Panel(self, -1, wx.DefaultPosition, wx.DefaultSize, style = wx.RAISED_BORDER | wx.TAB_TRAVERSAL)
-		fields_pnl.SetBackgroundColour(wx.Color(222,222,222))
+		fields_pnl.SetBackgroundColour(wx.Colour(222,222,222))
 		# rows, cols, hgap, vgap
 		gszr = wx.GridSizer(len(_prompt_defs[self._type]), 1, 2, 2)
 
@@ -1318,81 +1342,6 @@ class gmEditArea(cEditArea):
 				pass
 		return values
 #====================================================================
-class gmFamilyHxEditArea(gmEditArea):
-	def __init__(self, parent, id):
-		try:
-			gmEditArea.__init__(self, parent, id, aType = 'family history')
-		except gmExceptions.ConstructorError:
-			_log.exceptions('cannot instantiate family Hx edit area')
-			raise
-	#----------------------------------------------------------------
-	def _make_edit_lines(self, parent):
-		_log.debug("making family Hx lines")
-		lines = []
-		self.input_fields = {}
-		# line 1
-		# FIXME: put patient search widget here, too ...
-		# add button "make active patient"
-		self.input_fields['name'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		self.input_fields['DOB'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		lbl_dob = self._make_prompt(parent, _(" Date of Birth "), richards_blue)
-		szr = wx.BoxSizer(wx.HORIZONTAL)
-		szr.Add(self.input_fields['name'], 4, wx.EXPAND)
-		szr.Add(lbl_dob, 2, wx.EXPAND)
-		szr.Add(self.input_fields['DOB'], 4, wx.EXPAND)
-		lines.append(szr)
-		# line 2
-		# FIXME: keep relationship attachments permamently ! (may need to make new patient ...)
-		# FIXME: learning phrasewheel attached to list loaded from backend
-		self.input_fields['relationship'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		szr = wx.BoxSizer(wx.HORIZONTAL)
-		szr.Add(self.input_fields['relationship'], 4, wx.EXPAND)
-		lines.append(szr)
-		# line 3
-		self.input_fields['condition'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		self.cb_condition_confidential = wx.CheckBox(parent, -1, _("confidental"), wx.DefaultPosition, wx.DefaultSize, wx.NO_BORDER)
-		szr = wx.BoxSizer(wx.HORIZONTAL)
-		szr.Add(self.input_fields['condition'], 6, wx.EXPAND)
-		szr.Add(self.cb_condition_confidential, 0, wx.EXPAND)
-		lines.append(szr)
-		# line 4
-		self.input_fields['comment'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		lines.append(self.input_fields['comment'])
-		# line 5
-		lbl_onset = self._make_prompt(parent, _(" age onset "), richards_blue)
-		self.input_fields['age onset'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		#    FIXME: combo box ...
-		lbl_caused_death = self._make_prompt(parent, _(" caused death "), richards_blue)
-		self.input_fields['caused death'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		lbl_aod = self._make_prompt(parent, _(" age died "), richards_blue)
-		self.input_fields['AOD'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		szr = wx.BoxSizer(wx.HORIZONTAL)
-		szr.Add(lbl_onset, 0, wx.EXPAND)
-		szr.Add(self.input_fields['age onset'], 1,wx.EXPAND)
-		szr.Add(lbl_caused_death, 0, wx.EXPAND)
-		szr.Add(self.input_fields['caused death'], 2,wx.EXPAND)
-		szr.Add(lbl_aod, 0, wx.EXPAND)
-		szr.Add(self.input_fields['AOD'], 1, wx.EXPAND)
-		szr.Add(2, 2, 8)
-		lines.append(szr)
-		# line 6
-		self.input_fields['progress notes'] = cEditAreaField(parent, -1, wx.DefaultPosition, wx.DefaultSize)
-		lines.append(self.input_fields['progress notes'])
-		# line 8
-		self.Btn_next_condition = wx.Button(parent, -1, _("Next Condition"))
-		szr = wx.BoxSizer(wx.HORIZONTAL)
-		szr.AddSpacer(10, 0, 0)
-		szr.Add(self.Btn_next_condition, 0, wx.EXPAND | wx.ALL, 1)
-		szr.Add(2, 1, 5)
-		szr.Add(self._make_standard_buttons(parent), 0, wx.EXPAND)
-		lines.append(szr)
-
-		return lines
-
-	def _save_data(self):
-		return 1
-
-#====================================================================
 class gmPastHistoryEditArea(gmEditArea):
 
 	def __init__(self, parent, id):
@@ -1433,7 +1382,7 @@ class gmPastHistoryEditArea(gmEditArea):
 			widget = self.fld_age_noted,
 			weight = 2
 		)
-		
+
 		# line 2
 		self.fld_laterality_none= wx.RadioButton(parent, -1, _("N/A"))
 		self.fld_laterality_left= wx.RadioButton(parent, -1, _("L"))
@@ -1832,7 +1781,7 @@ class gmPnlEditAreaPrompts(wx.Panel):
 class EditTextBoxes(wx.Panel):
 	def __init__(self, parent, id, editareaprompts, section):
 		wx.Panel.__init__(self, parent, id, wx.DefaultPosition, wx.DefaultSize,style = wx.RAISED_BORDER | wx.TAB_TRAVERSAL)
-		self.SetBackgroundColour(wx.Color(222,222,222))
+		self.SetBackgroundColour(wx.Colour(222,222,222))
 		self.parent = parent
 		# rows, cols, hgap, vgap
 		self.gszr = wx.GridSizer(len(editareaprompts), 1, 2, 2)
@@ -1933,7 +1882,7 @@ class EditArea(wx.Panel):
 		_log.warning('***** old style EditArea instantiated, please convert *****')
 
 		wx.Panel.__init__(self, parent, id, wx.DefaultPosition, wx.DefaultSize, style = wx.NO_BORDER)
-		self.SetBackgroundColour(wx.Color(222,222,222))
+		self.SetBackgroundColour(wx.Colour(222,222,222))
 
 		# make prompts
 		prompts = gmPnlEditAreaPrompts(self, -1, line_labels)
@@ -2117,7 +2066,7 @@ class EditArea(wx.Panel):
 #	        elif section == gmSECTION_MEASUREMENTS:
 #		      self.combo_measurement_type = wx.ComboBox(self, ID_MEASUREMENT_TYPE, "", wx.DefaultPosition,wx.DefaultSize, ['Blood pressure','INR','Height','Weight','Whatever other measurement you want to put in here'], wx.CB_DROPDOWN)
 #		      self.combo_measurement_type.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL, wx.BOLD,False,''))
-#		      self.combo_measurement_type.SetForegroundColour(wx.Color(255,0,0))
+#		      self.combo_measurement_type.SetForegroundColour(wx.Colour(255,0,0))
 #		      self.txt_measurement_value = cEditAreaField(self,ID_MEASUREMENT_VALUE,wx.DefaultPosition,wx.DefaultSize)
 #		      self.txt_txt_measurement_date = cEditAreaField(self,ID_MEASUREMENT_DATE,wx.DefaultPosition,wx.DefaultSize)
 #		      self.txt_txt_measurement_comment = cEditAreaField(self,ID_MEASUREMENT_COMMENT,wx.DefaultPosition,wx.DefaultSize)
@@ -2220,7 +2169,7 @@ class EditArea(wx.Panel):
 #		      self.gszr.AddSizer(self.sizer_line10,0,wx.EXPAND)                   #e.g check boxes to include medications etc
 #		      self.gszr.Add(self.sizer_line11,0,wx.EXPAND)                       #e.g check boxes to include active problems etc
 		      #self.spacer = wxWindow(self,-1,wx.DefaultPosition,wx.DefaultSize)
-		      #self.spacer.SetBackgroundColour(wx.Color(255,255,255))
+		      #self.spacer.SetBackgroundColour(wx.Colour(255,255,255))
 #		      self.sizer_line12.Add(5,0,6)
 		      #self.sizer_line12.Add(self.spacer,6,wx.EXPAND)
 #		      self.sizer_line12.Add(self.btnpreview,1,wx.EXPAND|wxALL,2)
@@ -2232,15 +2181,15 @@ class EditArea(wx.Panel):
 		      #FIXME remove present options in this combo box	  #FIXME defaults need to be loaded from database	  
 #		      self.combo_tosee = wx.ComboBox(self, ID_RECALLS_TOSEE, "", wx.DefaultPosition,wx.DefaultSize, ['Doctor1','Doctor2','Nurse1','Dietition'], wx.CB_READONLY ) #wx.CB_DROPDOWN)
 #		      self.combo_tosee.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL, wx.BOLD,False,''))
-#		      self.combo_tosee.SetForegroundColour(wx.Color(255,0,0))
+#		      self.combo_tosee.SetForegroundColour(wx.Colour(255,0,0))
 		      #FIXME defaults need to be loaded from database
 #		      self.combo_recall_method = wx.ComboBox(self, ID_RECALLS_CONTACTMETHOD, "", wx.DefaultPosition,wx.DefaultSize, ['Letter','Telephone','Email','Carrier pigeon'], wx.CB_READONLY )
 #		      self.combo_recall_method.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL, wx.BOLD,False,''))
-#		      self.combo_recall_method.SetForegroundColour(wx.Color(255,0,0))
+#		      self.combo_recall_method.SetForegroundColour(wx.Colour(255,0,0))
 		      #FIXME defaults need to be loaded from database
  #                     self.combo_apptlength = wx.ComboBox(self, ID_RECALLS_APPNTLENGTH, "", wx.DefaultPosition,wx.DefaultSize, ['brief','standard','long','prolonged'], wx.CB_READONLY )
 #		      self.combo_apptlength.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL, wx.BOLD,False,''))
-#		      self.combo_apptlength.SetForegroundColour(wx.Color(255,0,0))
+#		      self.combo_apptlength.SetForegroundColour(wx.Colour(255,0,0))
 #		      self.txt_recall_for = cEditAreaField(self,ID_RECALLS_TXT_FOR, wx.DefaultPosition,wx.DefaultSize)
 #		      self.txt_recall_due = cEditAreaField(self,ID_RECALLS_TXT_DATEDUE, wx.DefaultPosition,wx.DefaultSize)
 #		      self.txt_recall_addtext = cEditAreaField(self,ID_RECALLS_TXT_ADDTEXT,wx.DefaultPosition,wx.DefaultSize)
