@@ -1,10 +1,10 @@
 """GNUmed provider inbox handling widgets.
 """
 #================================================================
-__version__ = "$Revision: 1.48 $"
 __author__ = "Karsten Hilbert <Karsten.Hilbert@gmx.net>"
 
-import sys, logging
+import sys
+import logging
 
 
 import wx
@@ -19,10 +19,14 @@ from Gnumed.pycommon import gmCfg
 from Gnumed.pycommon import gmTools
 from Gnumed.pycommon import gmDispatcher
 from Gnumed.pycommon import gmMatchProvider
+from Gnumed.pycommon import gmDateTime
+from Gnumed.pycommon import gmNetworkTools
 
 from Gnumed.business import gmPerson
+from Gnumed.business import gmStaff
 from Gnumed.business import gmSurgery
 from Gnumed.business import gmProviderInbox
+from Gnumed.business import gmClinicalRecord
 
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmListWidgets
@@ -34,10 +38,10 @@ from Gnumed.wxpython import gmAuthWidgets
 from Gnumed.wxpython import gmPatSearchWidgets
 from Gnumed.wxpython import gmVaccWidgets
 from Gnumed.wxpython import gmCfgWidgets
+from Gnumed.wxpython import gmDataPackWidgets
 
 
 _log = logging.getLogger('gm.ui')
-_log.info(__version__)
 
 _indicator = {
 	-1: '',
@@ -45,174 +49,12 @@ _indicator = {
 	1: '*!!*'
 }
 #============================================================
-from Gnumed.wxGladeWidgets import wxgTextExpansionEditAreaPnl
-
-class cTextExpansionEditAreaPnl(wxgTextExpansionEditAreaPnl.wxgTextExpansionEditAreaPnl, gmEditArea.cGenericEditAreaMixin):
-
-	def __init__(self, *args, **kwds):
-
-		try:
-			data = kwds['keyword']
-			del kwds['keyword']
-		except KeyError:
-			data = None
-
-		wxgTextExpansionEditAreaPnl.wxgTextExpansionEditAreaPnl.__init__(self, *args, **kwds)
-		gmEditArea.cGenericEditAreaMixin.__init__(self)
-
-		self.mode = 'new'
-		self.data = data
-		if data is not None:
-			self.mode = 'edit'
-
-		#self.__init_ui()
-		self.__register_interests()
-	#--------------------------------------------------------
-	def __init_ui(self, keyword=None):
-
-		if keyword is not None:
-			self.data = keyword
-	#----------------------------------------------------------------
-	# generic Edit Area mixin API
-	#----------------------------------------------------------------
-	def _valid_for_save(self):
-		validity = True
-
-		if self._TCTRL_keyword.GetValue().strip() == u'':
-			validity = False
-			self.display_tctrl_as_valid(tctrl = self._TCTRL_keyword, valid = False)
-			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save text expansion without keyword.'), beep = True)
-		else:
-			self.display_tctrl_as_valid(tctrl = self._TCTRL_keyword, valid = True)
-
-		if self._TCTRL_expansion.GetValue().strip() == u'':
-			validity = False
-			self.display_tctrl_as_valid(tctrl = self._TCTRL_expansion, valid = False)
-			gmDispatcher.send(signal = 'statustext', msg = _('Cannot save text expansion without expansion text.'), beep = True)
-		else:
-			self.display_tctrl_as_valid(tctrl = self._TCTRL_expansion, valid = True)
-
-		return validity
-	#----------------------------------------------------------------
-	def _save_as_new(self):
-		kwd = self._TCTRL_keyword.GetValue().strip()
-		saved = gmPG2.add_text_expansion (
-			keyword = kwd,
-			expansion = self._TCTRL_expansion.GetValue(),
-			public = self._RBTN_public.GetValue()
-		)
-		if not saved:
-			return False
-
-		self.data = kwd
-		return True
-	#----------------------------------------------------------------
-	def _save_as_update(self):
-		kwd = self._TCTRL_keyword.GetValue().strip()
-		gmPG2.edit_text_expansion (
-			keyword = kwd,
-			expansion = self._TCTRL_expansion.GetValue()
-		)
-		self.data = kwd
-		return True
-	#----------------------------------------------------------------
-	def _refresh_as_new(self):
-		self._TCTRL_keyword.SetValue(u'')
-		self._TCTRL_keyword.Enable(True)
-		self._TCTRL_expansion.SetValue(u'')
-		self._TCTRL_expansion.Enable(False)
-		self._RBTN_public.Enable(True)
-		self._RBTN_private.Enable(True)
-		self._RBTN_public.SetValue(1)
-
-		self._TCTRL_keyword.SetFocus()
-	#----------------------------------------------------------------
-	def _refresh_as_new_from_existing(self):
-		self._TCTRL_keyword.SetValue(u'%s%s' % (self.data, _(u'___copy')))
-		self._TCTRL_keyword.Enable(True)
-		expansion = gmPG2.expand_keyword(keyword = self.data)
-		self._TCTRL_expansion.SetValue(gmTools.coalesce(expansion, u''))
-		self._TCTRL_expansion.Enable(True)
-		self._RBTN_public.Enable(True)
-		self._RBTN_private.Enable(True)
-		self._RBTN_public.SetValue(1)
-
-		self._TCTRL_keyword.SetFocus()
-	#----------------------------------------------------------------
-	def _refresh_from_existing(self):
-		self._TCTRL_keyword.SetValue(self.data)
-		self._TCTRL_keyword.Enable(False)
-		expansion = gmPG2.expand_keyword(keyword = self.data)
-		self._TCTRL_expansion.SetValue(gmTools.coalesce(expansion, u''))
-		self._TCTRL_expansion.Enable(True)
-		self._RBTN_public.Enable(False)
-		self._RBTN_private.Enable(False)
-
-		self._TCTRL_expansion.SetFocus()
-	#----------------------------------------------------------------
-	# event handling
-	#----------------------------------------------------------------
-	def __register_interests(self):
-		self._TCTRL_keyword.Bind(wx.EVT_TEXT, self._on_keyword_modified)
-	#----------------------------------------------------------------
-	def _on_keyword_modified(self, evt):
-		if self._TCTRL_keyword.GetValue().strip() == u'':
-			self._TCTRL_expansion.Enable(False)
-		else:
-			self._TCTRL_expansion.Enable(True)
-#============================================================
-def configure_keyword_text_expansion(parent=None):
-
-	if parent is None:
-		parent = wx.GetApp().GetTopWindow()
-
-	#----------------------
-	def delete(keyword=None):
-		gmPG2.delete_text_expansion(keyword = keyword)
-		return True
-	#----------------------
-	def edit(keyword=None):
-		ea = cTextExpansionEditAreaPnl(parent, -1, keyword = keyword)
-		dlg = gmEditArea.cGenericEditAreaDlg2(parent, -1, edit_area = ea)
-		dlg.SetTitle (
-			gmTools.coalesce(keyword, _('Adding text expansion'), _('Editing text expansion "%s"'))
-		)
-		if dlg.ShowModal() == wx.ID_OK:
-			return True
-
-		return False
-	#----------------------
-	def refresh(lctrl=None):
-		kwds = [ [
-				r[0],
-				gmTools.bool2subst(r[1], gmTools.u_checkmark_thick, u''),
-				gmTools.bool2subst(r[2], gmTools.u_checkmark_thick, u''),
-				r[3]
-			] for r in gmPG2.get_text_expansion_keywords()
-		]
-		data = [ r[0] for r in gmPG2.get_text_expansion_keywords() ]
-		lctrl.set_string_items(kwds)
-		lctrl.set_data(data)
-	#----------------------
-
-	gmListWidgets.get_choices_from_list (
-		parent = parent,
-		msg = _('\nSelect the keyword you want to edit !\n'),
-		caption = _('Editing keyword-based text expansions ...'),
-		columns = [_('Keyword'), _('Public'), _('Private'), _('Owner')],
-		single_selection = True,
-		edit_callback = edit,
-		new_callback = edit,
-		delete_callback = delete,
-		refresh_callback = refresh
-	)
-#============================================================
 def configure_fallback_primary_provider(parent=None):
 
 	if parent is None:
 		parent = wx.GetApp().GetTopWindow()
 
-	staff = gmPerson.get_staff_list()
+	staff = gmStaff.get_staff_list()
 	choices = [ [
 			s[u'short_alias'],
 			u'%s%s %s' % (
@@ -385,7 +227,7 @@ def configure_workplace_plugins(parent=None):
 		picker.set_columns(['Available plugins'], ['Active plugins'])
 		available_plugins = gmPlugin.get_installed_plugins(plugin_dir = 'gui')
 		picker.set_choices(available_plugins)
-		picker.set_picks(picks = curr_plugins)
+		picker.set_picks(picks = curr_plugins[:])
 		btn_pressed = picker.ShowModal()
 		if btn_pressed != wx.ID_OK:
 			picker.Destroy()
@@ -575,6 +417,67 @@ class cMessageTypePhraseWheel(gmPhraseWheel.cPhraseWheel):
 			data = gmProviderInbox.create_inbox_item_type(message_type = val)
 		)
 #====================================================================
+def _display_clinical_reminders():
+	wx.CallAfter(__display_clinical_reminders)
+
+gmDispatcher.connect(signal = u'post_patient_selection', receiver = _display_clinical_reminders)
+
+def __display_clinical_reminders():
+	pat = gmPerson.gmCurrentPatient()
+	if not pat.connected:
+		return
+	for msg in pat.overdue_messages:
+		if msg['expiry_date'] is None:
+			exp = u''
+		else:
+			exp = _(' - expires %s') % gmDateTime.pydt_strftime (
+				msg['expiry_date'],
+				'%Y %b %d',
+				accuracy = gmDateTime.acc_days
+			)
+		txt = _(
+			'Due for %s (since %s%s):\n'
+			'%s'
+			'%s'
+			'\n'
+			'Patient: %s\n'
+			'Reminder by: %s'
+		) % (
+			gmDateTime.format_interval_medically(msg['interval_due']),
+			gmDateTime.pydt_strftime(msg['due_date'], '%Y %b %d', accuracy = gmDateTime.acc_days),
+			exp,
+			gmTools.coalesce(msg['comment'], u'', u'\n%s\n'),
+			gmTools.coalesce(msg['data'], u'', u'\n%s\n'),
+			pat['description_gender'],
+			msg['modified_by']
+		)
+		gmGuiHelpers.gm_show_warning (
+			aTitle = _('Clinical reminder'),
+			aMessage = txt
+		)
+	for hint in pat.dynamic_hints:
+		txt = u'%s\n\n%s\n\n          %s' % (
+			hint['title'],
+			gmTools.wrap(hint['hint'], width = 50, initial_indent = u' ', subsequent_indent = u' '),
+			hint['source']
+		)
+		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
+			None,
+			-1,
+			caption = _('Clinical hint'),
+			question = txt,
+			button_defs = [
+				{'label': _('OK'), 'tooltip': _('OK'), 'default': True},
+				{'label': _('More info'), 'tooltip': _('Go to [%s]') % hint['url']}
+			]
+		)
+		button = dlg.ShowModal()
+		dlg.Destroy()
+		if button == wx.ID_NO:
+			gmNetworkTools.open_url_in_browser(hint['url'], autoraise = False)
+
+	return
+#====================================================================
 from Gnumed.wxGladeWidgets import wxgInboxMessageEAPnl
 
 class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.cGenericEditAreaMixin):
@@ -646,6 +549,30 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 			self.display_ctrl_as_valid(ctrl = self._CHBOX_active_patient, valid = True)
 			self.display_ctrl_as_valid(ctrl = self._PRW_patient, valid = True)
 
+		if self._PRW_due.is_valid_timestamp(allow_empty = True):
+			self._PRW_due.display_as_valid(True)
+		else:
+			# non-empty but also not valid
+			validity = False
+			self._PRW_due.display_as_valid(False)
+
+		if self._PRW_expiry.is_valid_timestamp(allow_empty = True):
+			self._PRW_expiry.display_as_valid(True)
+		else:
+			# non-empty but also not valid
+			validity = False
+			self._PRW_expiry.display_as_valid(False)
+
+		# if .due is not empty AND valid
+		if self._PRW_due.is_valid_timestamp(allow_empty = False):
+			# and if .expiry is ALSO not empty AND valid
+			if self._PRW_expiry.is_valid_timestamp(allow_empty = False):
+				# only then compare .due and .expiry
+				if not self._PRW_expiry.date > self._PRW_due.date:
+					validity = False
+					self._PRW_expiry.display_as_valid(False)
+					gmDispatcher.send(signal = 'statustext', msg = _('Message cannot expire before being due.'))
+
 		return validity
 	#----------------------------------------------------------------
 	def _save_as_new(self):
@@ -659,7 +586,7 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 
 		receiver = None
 		if self._CHBOX_send_to_me.IsChecked():
-			receiver = gmPerson.gmCurrentProvider()['pk_staff']
+			receiver = gmStaff.gmCurrentProvider()['pk_staff']
 		else:
 			if self._PRW_receiver.GetData() is not None:
 				receiver = self._PRW_receiver.GetData()
@@ -673,6 +600,12 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 
 		msg['data'] = self._TCTRL_message.GetValue().strip()
 
+		if self._PRW_due.is_valid_timestamp():
+			msg['due_date'] = self._PRW_due.date
+
+		if self._PRW_expiry.is_valid_timestamp():
+			msg['expiry_date'] = self._PRW_expiry.date
+
 		if self._RBTN_normal.GetValue() is True:
 			msg['importance'] = 0
 		elif self._RBTN_high.GetValue() is True:
@@ -690,7 +623,7 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 		self.data['pk_type'] = self._PRW_type.GetData(can_create = True)
 
 		if self._CHBOX_send_to_me.IsChecked():
-			self.data['pk_staff'] = gmPerson.gmCurrentProvider()['pk_staff']
+			self.data['pk_staff'] = gmStaff.gmCurrentProvider()['pk_staff']
 		else:
 			self.data['pk_staff'] = self._PRW_receiver.GetData()
 
@@ -704,6 +637,12 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 			else:
 				self.data['pk_patient'] = self._PRW_patient.person.ID
 
+		if self._PRW_due.is_valid_timestamp():
+			self.data['due_date'] = self._PRW_due.date
+
+		if self._PRW_expiry.is_valid_timestamp():
+			self.data['expiry_date'] = self._PRW_expiry.date
+
 		if self._RBTN_normal.GetValue() is True:
 			self.data['importance'] = 0
 		elif self._RBTN_high.GetValue() is True:
@@ -719,8 +658,10 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 		self._PRW_type.SetText(value = u'', data = None)
 		self._CHBOX_send_to_me.SetValue(True)
 		self._PRW_receiver.Enable(False)
-		self._PRW_receiver.SetData(data = gmPerson.gmCurrentProvider()['pk_staff'])
+		self._PRW_receiver.SetData(data = gmStaff.gmCurrentProvider()['pk_staff'])
 		self._TCTRL_message.SetValue(u'')
+		self._PRW_due.SetText(data = None)
+		self._PRW_expiry.SetText(data = None)
 		self._RBTN_normal.SetValue(True)
 		self._RBTN_high.SetValue(False)
 		self._RBTN_low.SetValue(False)
@@ -746,13 +687,13 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 		self._TCTRL_subject.SetValue(gmTools.coalesce(self.data['comment'], u''))
 		self._PRW_type.SetData(data = self.data['pk_type'])
 
-		curr_prov = gmPerson.gmCurrentProvider()
+		curr_prov = gmStaff.gmCurrentProvider()
 		curr_pat = gmPerson.gmCurrentPatient()
 
 		if curr_prov['pk_staff'] == self.data['pk_staff']:
 			self._CHBOX_send_to_me.SetValue(True)
 			self._PRW_receiver.Enable(False)
-			self._PRW_receiver.SetData(data = gmPerson.gmCurrentProvider()['pk_staff'])
+			self._PRW_receiver.SetData(data = gmStaff.gmCurrentProvider()['pk_staff'])
 		else:
 			self._CHBOX_send_to_me.SetValue(False)
 			self._PRW_receiver.Enable(True)
@@ -782,6 +723,9 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 			else:
 				self._PRW_patient.person = gmPerson.cIdentity(aPK_obj = self.data['pk_patient'])
 
+		self._PRW_due.SetText(data = self.data['due_date'])
+		self._PRW_expiry.SetText(data = self.data['expiry_date'])
+
 		self._RBTN_normal.SetValue(False)
 		self._RBTN_high.SetValue(False)
 		self._RBTN_low.SetValue(False)
@@ -804,7 +748,7 @@ class cInboxMessageEAPnl(wxgInboxMessageEAPnl.wxgInboxMessageEAPnl, gmEditArea.c
 	def _on_send_to_me_checked(self, event):
 		if self._CHBOX_send_to_me.IsChecked():
 			self._PRW_receiver.Enable(False)
-			self._PRW_receiver.SetData(data = gmPerson.gmCurrentProvider()['pk_staff'])
+			self._PRW_receiver.SetData(data = gmStaff.gmCurrentProvider()['pk_staff'])
 		else:
 			self._PRW_receiver.Enable(True)
 			self._PRW_receiver.SetText(value = u'', data = None)
@@ -824,6 +768,44 @@ def edit_inbox_message(parent=None, message=None, single_entry=True):
 		return True
 	dlg.Destroy()
 	return False
+
+#============================================================
+def manage_reminders(parent=None, patient=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+	#------------------------------------------------------------
+	def refresh(lctrl):
+		reminders = gmProviderInbox.get_reminders(pk_patient = patient)
+		items = [ [
+			gmTools.bool2subst (
+				r['is_overdue'],
+				_('overdue for %s'),
+				_('due in %s')
+			) % gmDateTime.format_interval_medically(r['interval_due']),
+			r['comment'],
+			r['pk_inbox_message']
+		] for r in reminders ]
+		lctrl.set_string_items(items)
+		lctrl.set_data(reminders)
+	#------------------------------------------------------------
+	return gmListWidgets.get_choices_from_list (
+		parent = parent,
+		msg = None,
+		caption = _('Reminders for the current patient'),
+		columns = [ _('Status'), _('Subject'), '#' ],
+		single_selection = False,
+		can_return_empty = True,
+		ignore_OK_button = False,
+		refresh_callback = refresh
+#		edit_callback=None,
+#		new_callback=None,
+#		delete_callback=None,
+#		left_extra_button=None,
+#		middle_extra_button=None,
+#		right_extra_button=None
+	)
+
 #============================================================
 from Gnumed.wxGladeWidgets import wxgProviderInboxPnl
 
@@ -837,8 +819,7 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		wxgProviderInboxPnl.wxgProviderInboxPnl.__init__(self, *args, **kwds)
 		gmRegetMixin.cRegetOnPaintMixin.__init__(self)
 
-		self.provider = gmPerson.gmCurrentProvider()
-		self.filter_mode = 'all'
+		self.provider = gmStaff.gmCurrentProvider()
 		self.__init_ui()
 
 		cProviderInboxPnl._item_handlers['clinical.review docs'] = self._goto_doc_review
@@ -850,76 +831,210 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 	#--------------------------------------------------------
 	# reget-on-paint API
 	#--------------------------------------------------------
+	def _schedule_data_reget(self):
+		_log.debug('called by reget-on-paint mixin API')
+		gmRegetMixin.cRegetOnPaintMixin._schedule_data_reget(self)
+	#--------------------------------------------------------
 	def _populate_with_data(self):
+		_log.debug('_populate_with_data() (after _schedule_data_reget ?)')
 		self.__populate_inbox()
 		return True
 	#--------------------------------------------------------
+	# notebook plugin API
+	#--------------------------------------------------------
+	def repopulate_ui(self):
+		_log.debug('called by notebook plugin API, skipping inbox loading')
+		#gmRegetMixin.cRegetOnPaintMixin.repopulate_ui(self)
+		return True
+	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
 	def __register_interests(self):
-		gmDispatcher.connect(signal = u'message_inbox_generic_mod_db', receiver = self._on_message_inbox_mod_db)
+		gmDispatcher.connect(signal = u'message_inbox_generic_mod_db', receiver = self._on_message_inbox_generic_mod_db)
 		gmDispatcher.connect(signal = u'message_inbox_mod_db', receiver = self._on_message_inbox_mod_db)
 		# FIXME: listen for results insertion/deletion
-		gmDispatcher.connect(signal = u'reviewed_test_results_mod_db', receiver = self._on_message_inbox_mod_db)
-		gmDispatcher.connect(signal = u'identity_mod_db', receiver = self._on_message_inbox_mod_db)
-		gmDispatcher.connect(signal = u'doc_mod_db', receiver = self._on_message_inbox_mod_db)
-		gmDispatcher.connect(signal = u'doc_obj_review_mod_db', receiver = self._on_message_inbox_mod_db)
+		gmDispatcher.connect(signal = u'reviewed_test_results_mod_db', receiver = self._on_results_mod_db)
+		gmDispatcher.connect(signal = u'identity_mod_db', receiver = self._on_identity_mod_db)
+		gmDispatcher.connect(signal = u'doc_mod_db', receiver = self._on_doc_mod_db)
+		gmDispatcher.connect(signal = u'doc_obj_review_mod_db', receiver = self._on_doc_obj_review_mod_db)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 	#--------------------------------------------------------
 	def __init_ui(self):
-		self._LCTRL_provider_inbox.set_columns([u'', _('Sent'), _('Category'), _('Type'), _('Message')])
+		self._LCTRL_provider_inbox.debug = 'provider inbox list'
 
-		msg = _('\n Inbox of %(title)s %(lname)s.\n') % {
-			'title': gmTools.coalesce (
+		self._LCTRL_provider_inbox.set_columns([u'', _('Sent'), _('Status'), _('Category - Type'), _('Message')])
+		self._LCTRL_provider_inbox.item_tooltip_callback = self._get_msg_tooltip
+
+		self.__update_greeting()
+
+		if gmPerson.gmCurrentPatient().connected:
+			self._CHBOX_active_patient.Enable()
+	#--------------------------------------------------------
+	def __update_greeting(self, no_of_messages=None):
+		msg = _(' Inbox of %s %s%s') % (
+			gmTools.coalesce (
 				self.provider['title'],
 				gmPerson.map_gender2salutation(self.provider['gender'])
 			),
-			'lname': self.provider['lastnames']
-		}
-
-		self._LCTRL_provider_inbox.item_tooltip_callback = self._get_msg_tooltip
-
+			self.provider['lastnames'],
+			gmTools.coalesce(no_of_messages, u'.', _(': %s message(s)'))
+		)
 		self._msg_welcome.SetLabel(msg)
-
-		if gmPerson.gmCurrentPatient().connected:
-			self._RBTN_active_patient.Enable()
 	#--------------------------------------------------------
 	def __populate_inbox(self):
-		self.__msgs = self.provider.inbox.messages
+		_log.debug('populating provider inbox')
 
-		if self.filter_mode == 'active':
-			if gmPerson.gmCurrentPatient().connected:
-				curr_pat_id = gmPerson.gmCurrentPatient().ID
-				self.__msgs = [ m for m in self.__msgs if m['pk_patient'] == curr_pat_id ]
+		# calculate constraints
+		pk_patient = None
+		if self._CHBOX_active_patient.IsChecked():
+			_log.debug('restricting to active patient')
+			curr_pat = gmPerson.gmCurrentPatient()
+			if curr_pat.connected:
+				pk_patient = curr_pat.ID
+
+		include_without_provider = True
+		if self._CHBOX_active_provider.IsChecked():
+			_log.debug('restricting to active provider directly')
+			include_without_provider = False
+
+		# get which messages to show
+		if self._RBTN_all_messages.GetValue():
+			_log.debug('loading all but expired messages')
+			self.__msgs = self.provider.inbox.get_messages (
+				pk_patient = pk_patient,
+				include_without_provider = include_without_provider,
+				exclude_expired = True,
+				expired_only = False,
+				overdue_only = False,
+				unscheduled_only = False,
+				exclude_unscheduled = False
+			)
+		elif self._RBTN_overdue_messages.GetValue():
+			_log.debug('loading overdue messages only')
+			self.__msgs = self.provider.inbox.get_messages (
+				pk_patient = pk_patient,
+				include_without_provider = include_without_provider,
+				exclude_expired = True,
+				expired_only = False,
+				overdue_only = True,
+				unscheduled_only = False,
+				exclude_unscheduled = True,
+				order_by = u'due_date, importance DESC, received_when DESC'
+			)
+		elif self._RBTN_scheduled_messages.GetValue():
+			_log.debug('loading overdue messages only')
+			self.__msgs = self.provider.inbox.get_messages (
+				pk_patient = pk_patient,
+				include_without_provider = include_without_provider,
+				exclude_expired = True,
+				expired_only = False,
+				overdue_only = False,
+				unscheduled_only = False,
+				exclude_unscheduled = True,
+				order_by = u'due_date, importance DESC, received_when DESC'
+			)
+		elif self._RBTN_unscheduled_messages.GetValue():
+			_log.debug('loading unscheduled messages only')
+			self.__msgs = self.provider.inbox.get_messages (
+				pk_patient = pk_patient,
+				include_without_provider = include_without_provider,
+				exclude_expired = True,
+				expired_only = False,
+				overdue_only = False,
+				unscheduled_only = True,
+				exclude_unscheduled = False
+			)
+		elif self._RBTN_expired_messages.GetValue():
+			_log.debug('loading expired messages only')
+			self.__msgs = self.provider.inbox.get_messages (
+				pk_patient = pk_patient,
+				include_without_provider = include_without_provider,
+				exclude_expired = False,
+				expired_only = True,
+				overdue_only = False,
+				unscheduled_only = False,
+				exclude_unscheduled = True,
+				order_by = u'expiry_date DESC, importance DESC, received_when DESC'
+			)
+
+		_log.debug('total # of inbox msgs: %s', len(self.__msgs))
+
+		items = []
+		for m in self.__msgs:
+			item = [_indicator[m['importance']], gmDateTime.pydt_strftime(m['received_when'], '%Y-%m-%d')]
+			if m['due_date'] is None:
+				item.append(u'')
 			else:
-				self.__msgs = []
+				if m['is_expired'] is True:
+					item.append(_('expired'))
+				else:
+					if m['is_overdue'] is True:
+						item.append(_('%s overdue') % gmDateTime.format_interval_medically(m['interval_due']))
+					else:
+						item.append(_('due in %s') % gmDateTime.format_interval_medically(m['interval_due']))
+			item.append(u'%s - %s' % (m['l10n_category'], m['l10n_type']))
+			item.append(m['comment'])
+			items.append(item)
 
-		items = [
-			[
-				_indicator[m['importance']],
-				m['received_when'].strftime('%Y-%m-%d'),
-				m['l10n_category'],
-				m['l10n_type'],
-				m['comment']
-			] for m in self.__msgs
-		]
+		_log.debug('# of list items created from msgs: %s', len(items))
 		self._LCTRL_provider_inbox.set_string_items(items = items)
 		self._LCTRL_provider_inbox.set_data(data = self.__msgs)
 		self._LCTRL_provider_inbox.set_column_widths()
 		self._TXT_inbox_item_comment.SetValue(u'')
+		self.__update_greeting(len(items))
 	#--------------------------------------------------------
 	# event handlers
 	#--------------------------------------------------------
-	def _on_post_patient_selection(self):
-		wx.CallAfter(self._schedule_data_reget)
-		wx.CallAfter(self._RBTN_active_patient.Enable)
+	def _on_results_mod_db(self):
+		_log.debug('reviewed_test_results_mod_db')
+		wx.CallAfter(self.__on_message_inbox_mod_db)
+	#--------------------------------------------------------
+	def _on_identity_mod_db(self):
+		_log.debug('identity_mod_db')
+		wx.CallAfter(self.__on_message_inbox_mod_db)
+	#--------------------------------------------------------
+	def _on_doc_obj_review_mod_db(self):
+		_log.debug('doc_obj_review_mod_db')
+		wx.CallAfter(self.__on_message_inbox_mod_db)
+	#--------------------------------------------------------
+	def _on_doc_mod_db(self):
+		_log.debug('doc_mod_db')
+		wx.CallAfter(self.__on_message_inbox_mod_db)
+	#--------------------------------------------------------
+	def _on_message_inbox_generic_mod_db(self, *args, **kwargs):
+		_log.debug('message_inbox_generic_mod_db')
+		wx.CallAfter(self.__on_message_inbox_mod_db)
 	#--------------------------------------------------------
 	def _on_message_inbox_mod_db(self, *args, **kwargs):
-		wx.CallAfter(self._schedule_data_reget)
+		_log.debug('message_inbox_mod_db')
+		wx.CallAfter(self.__on_message_inbox_mod_db)
+
+	def __on_message_inbox_mod_db(self):
+		self._schedule_data_reget()
 		gmDispatcher.send(signal = u'request_user_attention', msg = _('Please check your GNUmed Inbox !'))
 	#--------------------------------------------------------
+	def _on_post_patient_selection(self):
+		_log.debug('post_patient_selection')
+		wx.CallAfter(self.__on_post_patient_selection)
+
+	def __on_post_patient_selection(self):
+		self._CHBOX_active_patient.Enable()
+		self._schedule_data_reget()
+	#--------------------------------------------------------
 	def _lst_item_activated(self, evt):
-		msg = self._LCTRL_provider_inbox.get_selected_item_data(only_one = True)
+
+		try:
+			msg = self._LCTRL_provider_inbox.get_selected_item_data(only_one = True)
+		except IndexError:
+			_log.exception('problem with provider inbox item data access')
+			gmGuiHelpers.gm_show_error (
+				aTitle = _('handling provider inbox item'),
+				aMessage = _('There was a problem accessing the message data.')
+			)
+			_log.debug('effecting inbox reload')
+			wx.CallAfter(self.__populate_inbox)
+			return False
+
 		if msg is None:
 			return
 
@@ -996,55 +1111,26 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 		self.PopupMenu(menu, wx.DefaultPosition)
 		menu.Destroy()
 	#--------------------------------------------------------
-	def _on_all_messages_radiobutton_selected(self, event):
-		self.filter_mode = 'all'
+	def _on_message_range_radiobutton_selected(self, event):
 		self._TXT_inbox_item_comment.SetValue(u'')
+		_log.debug('_on_message_range_radiobutton_selected')
 		self.__populate_inbox()
 	#--------------------------------------------------------
-	def _on_active_patient_radiobutton_selected(self, event):
-		self.filter_mode = 'active'
+	def _on_active_patient_checkbox_ticked(self, event):
 		self._TXT_inbox_item_comment.SetValue(u'')
+		_log.debug('_on_active_patient_checkbox_ticked')
+		self.__populate_inbox()
+	#--------------------------------------------------------
+	def _on_active_provider_checkbox_ticked(self, event):
+		self._TXT_inbox_item_comment.SetValue(u'')
+		_log.debug('_on_active_provider_checkbox_ticked')
 		self.__populate_inbox()
 	#--------------------------------------------------------
 	def _on_add_button_pressed(self, event):
 		edit_inbox_message(parent = self, message = None, single_entry = False)
 	#--------------------------------------------------------
 	def _get_msg_tooltip(self, msg):
-		tt = u'%s: %s%s\n' % (
-			msg['received_when'].strftime('%A, %Y %B %d, %H:%M').decode(gmI18N.get_encoding()),
-			gmTools.bool2subst(msg['is_virtual'], _('virtual message'), _('message')),
-			gmTools.coalesce(msg['pk_inbox_message'], u'', u' #%s ')
-		)
-
-		tt += u'%s: %s\n' % (
-			msg['l10n_category'],
-			msg['l10n_type']
-		)
-
-		tt += u'%s %s %s\n' % (
-			msg['modified_by'],
-			gmTools.u_right_arrow,
-			gmTools.coalesce(msg['provider'], _('everyone'))
-		)
-
-		tt += u'\n%s%s%s\n\n' % (
-			gmTools.u_left_double_angle_quote,
-			msg['comment'],
-			gmTools.u_right_double_angle_quote
-		)
-
-		tt += gmTools.coalesce (
-			msg['pk_patient'],
-			u'',
-			u'%s\n\n' % _('Patient #%s')
-		)
-
-		if msg['data'] is not None:
-			tt += msg['data'][:150]
-			if len(msg['data']) > 150:
-				tt += gmTools.u_ellipsis
-
-		return tt
+		return msg.format()
 	#--------------------------------------------------------
 	# item handlers
 	#--------------------------------------------------------
@@ -1056,6 +1142,19 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 			gmDispatcher.send(signal = 'statustext', msg = _('You must deal with the reason for this message to remove it from your inbox.'), beep = True)
 			return False
 
+		pk_patient = self.__focussed_msg['pk_patient']
+		if pk_patient is not None:
+			emr = gmClinicalRecord.cClinicalRecord(aPKey = pk_patient, allow_user_interaction = False)
+			epi = emr.add_episode(episode_name = 'administration', is_open = False)
+			soap_cat = gmTools.bool2subst (
+				(self.__focussed_msg['category'] == u'clinical'),
+				u'U',
+				None
+			)
+			narr = _('Deleted inbox message:\n%s') % self.__focussed_msg.format(with_patient = False)
+			emr.add_clin_narrative(note = narr, soap_cat = soap_cat, episode = epi)
+			gmDispatcher.send(signal = 'statustext', msg = _('Recorded deletion of inbox message in EMR.'), beep = False)
+
 		if not self.provider.inbox.delete_message(self.__focussed_msg['pk_inbox_message']):
 			gmDispatcher.send(signal='statustext', msg=_('Problem removing message from Inbox.'))
 			return False
@@ -1190,6 +1289,62 @@ class cProviderInboxPnl(wxgProviderInboxPnl.wxgProviderInboxPnl, gmRegetMixin.cR
 
 		return True
 #============================================================
+def browse_dynamic_hints(parent=None):
+
+	if parent is None:
+		parent = wx.GetApp().GetTopWindow()
+	#------------------------------------------------------------
+	def get_tooltip(item):
+		if item is None:
+			return None
+		return item.format()
+	#------------------------------------------------------------
+	def switch_activation(item):
+		conn = gmAuthWidgets.get_dbowner_connection(procedure = _('Switching clinical hint activation'))
+		if conn is None:
+			return False
+		item['is_active'] = not item['is_active']
+		return item.save(conn = conn)
+	#------------------------------------------------------------
+	def manage_data_packs(item):
+		gmDataPackWidgets.manage_data_packs(parent = parent)
+		return True
+	#------------------------------------------------------------
+	def refresh(lctrl):
+		hints = gmProviderInbox.get_dynamic_hints(order_by = u'is_active DESC, source, hint')
+		items = [ [
+			gmTools.bool2subst(h['is_active'], gmTools.u_checkmark_thin, u''),
+			h['title'],
+			h['source'][:30],
+			h['hint'][:60],
+			gmTools.coalesce(h['url'], u'')[:60],
+			h['lang'],
+			h['pk']
+		] for h in hints ]
+		lctrl.set_string_items(items)
+		lctrl.set_data(hints)
+	#------------------------------------------------------------
+	gmListWidgets.get_choices_from_list (
+		parent = parent,
+		msg = _('\nDynamic hints registered with GNUmed.\n'),
+		caption = _('Showing dynamic hints.'),
+		columns = [ _('Active'), _('Title'), _('Source'), _('Hint'), u'URL', _('Language'), u'#' ],
+		single_selection = True,
+		refresh_callback = refresh,
+		left_extra_button = (
+			_('(De)-Activate'),
+			_('Switch activation of the selected hint'),
+			switch_activation
+		),
+		right_extra_button = (
+			_('Data packs'),
+			_('Browse and install clinical hints data packs'),
+			manage_data_packs
+		),
+		list_tooltip_callback = get_tooltip
+	)
+
+#============================================================
 if __name__ == '__main__':
 
 	if len(sys.argv) < 2:
