@@ -9,8 +9,10 @@ __license__ = 'GPL v2 or later (details at http://www.gnu.org)'
 import logging
 
 
-from Gnumed.wxpython import gmPlugin, gmMedicationWidgets
-from Gnumed.pycommon import gmI18N
+from Gnumed.wxpython import gmPlugin
+from Gnumed.wxpython import gmMedicationWidgets
+
+from Gnumed.wxpython import gmAccessPermissionWidgets
 
 
 _log = logging.getLogger('gm.ui')
@@ -19,6 +21,16 @@ class gmCurrentSubstancesPlugin(gmPlugin.cNotebookPlugin):
 	"""Plugin to encapsulate patient current medication list."""
 
 	tab_name = _('Medication')
+	required_minimum_role = 'doctor'
+
+	@gmAccessPermissionWidgets.verify_minimum_required_role (
+		required_minimum_role,
+		activity = _('loading plugin <%s>') % tab_name,
+		return_value_on_failure = False,
+		fail_silently = False
+	)
+	def register(self):
+		gmPlugin.cNotebookPlugin.register(self)
 
 	def name (self):
 		return gmCurrentSubstancesPlugin.tab_name
